// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
   
package it.esuli;

import android.app.Activity;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;

public class ShakeListener implements SensorEventListener {
	public interface OnShakeListener {
		boolean isItAShake(float[] orientation, float[] acceleration);

		void onShake(float[] orientation, float[] acceleration);
	}

	private long lastShake;
	private long minShakeInterval;

	private SensorManager sensorManager;
	private float[] orientation;
	private float[] acceleration;

	private OnShakeListener listener;

	public ShakeListener(Activity context) {
		sensorManager = (SensorManager) context
				.getSystemService(Activity.SENSOR_SERVICE);

		orientation = new float[3];
		acceleration = new float[3];

		minShakeInterval = 200;
		lastShake = 0;

		listener = null;
	}

	public void setOnShakeListener(OnShakeListener listener) {
		this.listener = listener;
	}

	public void setMinShakeInterval(long minShakeInterval) {
		this.minShakeInterval = minShakeInterval;
	}

	public float getMinShakeInterval() {
		return minShakeInterval;
	}

	public void onResume() {
		sensorManager.registerListener(this,
				sensorManager.getDefaultSensor(Sensor.TYPE_ORIENTATION),
				SensorManager.SENSOR_DELAY_GAME);
		sensorManager.registerListener(this,
				sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
				SensorManager.SENSOR_DELAY_GAME);
	}

	public void onAccuracyChanged(Sensor sensor, int accuracy) {
	}

	public void onSensorChanged(SensorEvent event) {
		if (listener != null) {
			if (event.sensor.getType() == Sensor.TYPE_ORIENTATION) {
				orientation[SensorManager.DATA_X] = event.values[SensorManager.DATA_X];
				orientation[SensorManager.DATA_Y] = event.values[SensorManager.DATA_Y];
				orientation[SensorManager.DATA_Z] = event.values[SensorManager.DATA_Z];

			} else if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
				acceleration[SensorManager.DATA_X] = event.values[SensorManager.DATA_X];
				acceleration[SensorManager.DATA_Y] = event.values[SensorManager.DATA_Y];
				acceleration[SensorManager.DATA_Z] = event.values[SensorManager.DATA_Z];

				if (listener.isItAShake(orientation, acceleration)) {
					long now = System.currentTimeMillis();
					if ((now - lastShake) > minShakeInterval) {
						listener.onShake(orientation, acceleration);
						lastShake = System.currentTimeMillis();
					}
				}
			}
		}
	}

	public void onPause() {
		sensorManager.unregisterListener(this);
	}

}
